#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class ApplicationCategory(Resource):
    """Provides the list of application categories that are available, to classify volumes depending on the applications that use them.

    # Parameters
    id             : Identifier for the application category.
    name           : Name of application category.
    dedupe_enabled : Specifies if dedupe is enabled for performance policies associated with this application category.
    creation_time  : Time when this application category was created.
    last_modified  : Time when this application category was last modified.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class ApplicationCategoryList(Collection):
    resource = ApplicationCategory
    resource_type = "application_categories"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")
