#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class UserGroup(Resource):
    """Represents Active Directory groups configured to manage the system.

    # Parameters
    id                 : Identifier for the user group.
    name               : Name of the user group.
    description        : Description of the user group.
    role_id            : Identifier for the user group's role.
    role               : Role of the user.
    inactivity_timeout : The amount of time that the user session is inactive before timing out. A value of 0 indicates that the timeout is taken from the group setting.
    creation_time      : Time when this user was created.
    last_modified      : Time when this user was last modified.
    disabled           : User is currently disabled.
    external_id        : External ID of the user group. In Active Directory, it is the group's SID (Security Identifier).
    domain_id          : Identifier of the domain this user group belongs to.
    domain_name        : Role of the user.
    """


class UserGroupList(Collection):
    resource = UserGroup
    resource_type = "user_groups"
