#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class SoftwareVersion(Resource):
    """Show the software version.

    # Parameters
    version                : Software version, used as identifier in URL.
    signature              : Keyed hash of download package.
    name                   : Name of version.
    status                 : Status of version.
    total_bytes            : Size of version.
    downloaded_bytes       : Number of bytes downloaded for the version.
    blacklist_reason       : Reason for blacklisting the version. Empty if version is not blacklisted.
    release_date           : Date when software version was released.
    is_manually_downloaded : Whether or not the version was downloaded manually.
    release_status         : Release status of software version.
    no_partial_response    : Indicate that it is not ok to provide partially available response.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class SoftwareVersionList(Collection):
    resource = SoftwareVersion
    resource_type = "software_versions"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")
