#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#

from ..query import _Field


class ApplicationServerFields():
    id = _Field('id')
    name = _Field('name')
    hostname = _Field('hostname')
    port = _Field('port')
    username = _Field('username')
    description = _Field('description')
    password = _Field('password')
    server_type = _Field('server_type')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')


class DiskFields():
    id = _Field('id')
    is_dfc = _Field('is_dfc')
    serial = _Field('serial')
    path = _Field('path')
    shelf_serial = _Field('shelf_serial')
    shelf_location = _Field('shelf_location')
    shelf_id = _Field('shelf_id')
    shelf_location_id = _Field('shelf_location_id')
    vshelf_id = _Field('vshelf_id')
    slot = _Field('slot')
    bank = _Field('bank')
    model = _Field('model')
    vendor = _Field('vendor')
    firmware_version = _Field('firmware_version')
    hba = _Field('hba')
    port = _Field('port')
    size = _Field('size')
    state = _Field('state')
    type = _Field('type')
    block_type = _Field('block_type')
    raid_id = _Field('raid_id')
    raid_resync_percent = _Field('raid_resync_percent')
    raid_resync_current_speed = _Field('raid_resync_current_speed')
    raid_resync_average_speed = _Field('raid_resync_average_speed')
    raid_state = _Field('raid_state')
    disk_internal_stat_1 = _Field('disk_internal_stat_1')
    smart_attribute_list = _Field('smart_attribute_list')
    disk_op = _Field('disk_op')
    force = _Field('force')
    array_name = _Field('array_name')
    array_id = _Field('array_id')
    partial_response_ok = _Field('partial_response_ok')


class FolderFields():
    id = _Field('id')
    name = _Field('name')
    fqn = _Field('fqn')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    limit_bytes_specified = _Field('limit_bytes_specified')
    limit_bytes = _Field('limit_bytes')
    limit_size_bytes = _Field('limit_size_bytes')
    provisioned_limit_size_bytes = _Field('provisioned_limit_size_bytes')
    overdraft_limit_pct = _Field('overdraft_limit_pct')
    capacity_bytes = _Field('capacity_bytes')
    free_space_bytes = _Field('free_space_bytes')
    provisioned_bytes = _Field('provisioned_bytes')
    usage_bytes = _Field('usage_bytes')
    volume_mapped_bytes = _Field('volume_mapped_bytes')
    usage_valid = _Field('usage_valid')
    agent_type = _Field('agent_type')
    inherited_vol_perfpol_id = _Field('inherited_vol_perfpol_id')
    inherited_vol_perfpol_name = _Field('inherited_vol_perfpol_name')
    unused_reserve_bytes = _Field('unused_reserve_bytes')
    unused_snap_reserve_bytes = _Field('unused_snap_reserve_bytes')
    compressed_vol_usage_bytes = _Field('compressed_vol_usage_bytes')
    compressed_snap_usage_bytes = _Field('compressed_snap_usage_bytes')
    uncompressed_vol_usage_bytes = _Field('uncompressed_vol_usage_bytes')
    uncompressed_snap_usage_bytes = _Field('uncompressed_snap_usage_bytes')
    vol_compression_ratio = _Field('vol_compression_ratio')
    snap_compression_ratio = _Field('snap_compression_ratio')
    compression_ratio = _Field('compression_ratio')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    num_snaps = _Field('num_snaps')
    num_snapcolls = _Field('num_snapcolls')
    app_uuid = _Field('app_uuid')
    volume_list = _Field('volume_list')
    appserver_id = _Field('appserver_id')
    appserver_name = _Field('appserver_name')
    folset_id = _Field('folset_id')
    folset_name = _Field('folset_name')
    limit_iops = _Field('limit_iops')
    limit_mbps = _Field('limit_mbps')
    access_protocol = _Field('access_protocol')
    tenant_id = _Field('tenant_id')


class InitiatorFields():
    id = _Field('id')
    access_protocol = _Field('access_protocol')
    initiator_group_id = _Field('initiator_group_id')
    initiator_group_name = _Field('initiator_group_name')
    label = _Field('label')
    iqn = _Field('iqn')
    ip_address = _Field('ip_address')
    alias = _Field('alias')
    chapuser_id = _Field('chapuser_id')
    wwpn = _Field('wwpn')
    vp_override = _Field('vp_override')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    override_existing_alias = _Field('override_existing_alias')


class ActiveDirectoryMembershipFields():
    id = _Field('id')
    description = _Field('description')
    name = _Field('name')
    netbios = _Field('netbios')
    server_list = _Field('server_list')
    computer_name = _Field('computer_name')
    organizational_unit = _Field('organizational_unit')
    user = _Field('user')
    password = _Field('password')
    enabled = _Field('enabled')


class PerformancePolicyFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    block_size = _Field('block_size')
    compress = _Field('compress')
    cache = _Field('cache')
    cache_policy = _Field('cache_policy')
    space_policy = _Field('space_policy')
    app_category = _Field('app_category')
    dedupe_enabled = _Field('dedupe_enabled')
    deprecated = _Field('deprecated')
    predefined = _Field('predefined')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    sample_rate = _Field('sample_rate')
    volume_count = _Field('volume_count')
    dedupe_override_pools = _Field('dedupe_override_pools')


class ChapUserFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    password = _Field('password')
    initiator_iqns = _Field('initiator_iqns')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    vol_list = _Field('vol_list')
    vol_count = _Field('vol_count')


class AuditLogFields():
    id = _Field('id')
    type = _Field('type')
    object_id = _Field('object_id')
    object_name = _Field('object_name')
    object_type = _Field('object_type')
    scope = _Field('scope')
    time = _Field('time')
    status = _Field('status')
    error_code = _Field('error_code')
    user_id = _Field('user_id')
    user_name = _Field('user_name')
    user_full_name = _Field('user_full_name')
    source_ip = _Field('source_ip')
    ext_user_id = _Field('ext_user_id')
    ext_user_group_id = _Field('ext_user_group_id')
    ext_user_group_name = _Field('ext_user_group_name')
    app_name = _Field('app_name')
    access_type = _Field('access_type')
    category = _Field('category')
    activity_type = _Field('activity_type')
    activity = _Field('activity')


class FibreChannelPortFields():
    id = _Field('id')
    array_name_or_serial = _Field('array_name_or_serial')
    controller_name = _Field('controller_name')
    fc_port_name = _Field('fc_port_name')
    bus_location = _Field('bus_location')
    port = _Field('port')
    slot = _Field('slot')
    orientation = _Field('orientation')
    link_info = _Field('link_info')
    rx_power = _Field('rx_power')
    tx_power = _Field('tx_power')


class FibreChannelConfigFields():
    id = _Field('id')
    array_list = _Field('array_list')
    group_leader_array = _Field('group_leader_array')


class SnapshotCollectionFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    volcoll_name = _Field('volcoll_name')
    volcoll_id = _Field('volcoll_id')
    origin_name = _Field('origin_name')
    is_replica = _Field('is_replica')
    srep_owner_name = _Field('srep_owner_name')
    srep_owner_id = _Field('srep_owner_id')
    peer_snapcoll_id = _Field('peer_snapcoll_id')
    is_complete = _Field('is_complete')
    is_manual = _Field('is_manual')
    is_external_trigger = _Field('is_external_trigger')
    is_unmanaged = _Field('is_unmanaged')
    is_manually_managed = _Field('is_manually_managed')
    repl_status = _Field('repl_status')
    repl_start_time = _Field('repl_start_time')
    repl_complete_time = _Field('repl_complete_time')
    repl_bytes_transferred = _Field('repl_bytes_transferred')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    online_status = _Field('online_status')
    vol_snap_attr_list = _Field('vol_snap_attr_list')
    snapshots_list = _Field('snapshots_list')
    replicate = _Field('replicate')
    replicate_to = _Field('replicate_to')
    start_online = _Field('start_online')
    allow_writes = _Field('allow_writes')
    disable_appsync = _Field('disable_appsync')
    snap_verify = _Field('snap_verify')
    skip_db_consistency_check = _Field('skip_db_consistency_check')
    sched_id = _Field('sched_id')
    sched_name = _Field('sched_name')
    invoke_on_upstream_partner = _Field('invoke_on_upstream_partner')
    agent_type = _Field('agent_type')
    expiry_after = _Field('expiry_after')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    force = _Field('force')


class TokenFields():
    id = _Field('id')
    session_token = _Field('session_token')
    username = _Field('username')
    password = _Field('password')
    app_name = _Field('app_name')
    sdk_name = _Field('sdk_name')
    source_ip = _Field('source_ip')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    expiry_time = _Field('expiry_time')
    server_uuid = _Field('server_uuid')


class ApplicationCategoryFields():
    id = _Field('id')
    name = _Field('name')
    dedupe_enabled = _Field('dedupe_enabled')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')


class ArrayFields():
    id = _Field('id')
    name = _Field('name')
    force = _Field('force')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    status = _Field('status')
    role = _Field('role')
    group_state = _Field('group_state')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    model = _Field('model')
    serial = _Field('serial')
    version = _Field('version')
    is_sfa = _Field('is_sfa')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    usage_valid = _Field('usage_valid')
    usable_capacity_bytes = _Field('usable_capacity_bytes')
    usable_cache_capacity_bytes = _Field('usable_cache_capacity_bytes')
    raw_capacity_bytes = _Field('raw_capacity_bytes')
    vol_usage_bytes = _Field('vol_usage_bytes')
    vol_usage_uncompressed_bytes = _Field('vol_usage_uncompressed_bytes')
    vol_compression = _Field('vol_compression')
    vol_saved_bytes = _Field('vol_saved_bytes')
    snap_usage_bytes = _Field('snap_usage_bytes')
    snap_usage_uncompressed_bytes = _Field('snap_usage_uncompressed_bytes')
    snap_compression = _Field('snap_compression')
    snap_space_reduction = _Field('snap_space_reduction')
    snap_saved_bytes = _Field('snap_saved_bytes')
    pending_delete_bytes = _Field('pending_delete_bytes')
    available_bytes = _Field('available_bytes')
    usage = _Field('usage')
    all_flash = _Field('all_flash')
    dedupe_capacity_bytes = _Field('dedupe_capacity_bytes')
    dedupe_usage_bytes = _Field('dedupe_usage_bytes')
    is_fully_dedupe_capable = _Field('is_fully_dedupe_capable')
    extended_model = _Field('extended_model')
    is_supported_hw_config = _Field('is_supported_hw_config')
    gig_nic_port_count = _Field('gig_nic_port_count')
    ten_gig_sfp_nic_port_count = _Field('ten_gig_sfp_nic_port_count')
    ten_gig_t_nic_port_count = _Field('ten_gig_t_nic_port_count')
    fc_port_count = _Field('fc_port_count')
    public_key = _Field('public_key')
    upgrade = _Field('upgrade')
    create_pool = _Field('create_pool')
    pool_description = _Field('pool_description')
    allow_lower_limits = _Field('allow_lower_limits')
    ctrlr_a_support_ip = _Field('ctrlr_a_support_ip')
    ctrlr_b_support_ip = _Field('ctrlr_b_support_ip')
    nic_list = _Field('nic_list')
    model_sub_type = _Field('model_sub_type')
    zconf_ipaddrs = _Field('zconf_ipaddrs')
    secondary_mgmt_ip = _Field('secondary_mgmt_ip')


class EventFields():
    id = _Field('id')
    type = _Field('type')
    name = _Field('name')
    scope = _Field('scope')
    target = _Field('target')
    target_type = _Field('target_type')
    timestamp = _Field('timestamp')
    category = _Field('category')
    severity = _Field('severity')
    summary = _Field('summary')
    activity = _Field('activity')
    alarm_id = _Field('alarm_id')
    params = _Field('params')
    tenant_id = _Field('tenant_id')


class SoftwareVersionFields():
    version = _Field('version')
    signature = _Field('signature')
    name = _Field('name')
    status = _Field('status')
    total_bytes = _Field('total_bytes')
    downloaded_bytes = _Field('downloaded_bytes')
    blacklist_reason = _Field('blacklist_reason')
    release_date = _Field('release_date')
    is_manually_downloaded = _Field('is_manually_downloaded')
    release_status = _Field('release_status')
    no_partial_response = _Field('no_partial_response')


class UserGroupFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    role_id = _Field('role_id')
    role = _Field('role')
    inactivity_timeout = _Field('inactivity_timeout')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    disabled = _Field('disabled')
    external_id = _Field('external_id')
    domain_id = _Field('domain_id')
    domain_name = _Field('domain_name')


class JobFields():
    completion_time = _Field('completion_time')
    creation_time = _Field('creation_time')
    current_phase = _Field('current_phase')
    current_phase_description = _Field('current_phase_description')
    description = _Field('description')
    id = _Field('id')
    name = _Field('name')
    last_modified = _Field('last_modified')
    object_id = _Field('object_id')
    op_type = _Field('op_type')
    type = _Field('type')
    parent_job_id = _Field('parent_job_id')
    percent_complete = _Field('percent_complete')
    request = _Field('request')
    response = _Field('response')
    state = _Field('state')
    result = _Field('result')
    total_phases = _Field('total_phases')


class ReplicationPartnerFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    partner_type = _Field('partner_type')
    alias = _Field('alias')
    secret = _Field('secret')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    control_port = _Field('control_port')
    hostname = _Field('hostname')
    port_range_start = _Field('port_range_start')
    proxy_hostname = _Field('proxy_hostname')
    proxy_user = _Field('proxy_user')
    repl_hostname = _Field('repl_hostname')
    data_port = _Field('data_port')
    is_alive = _Field('is_alive')
    partner_group_uid = _Field('partner_group_uid')
    last_keepalive_error = _Field('last_keepalive_error')
    cfg_sync_status = _Field('cfg_sync_status')
    last_sync_error = _Field('last_sync_error')
    array_serial = _Field('array_serial')
    version = _Field('version')
    pool_id = _Field('pool_id')
    pool_name = _Field('pool_name')
    folder_id = _Field('folder_id')
    folder_name = _Field('folder_name')
    match_folder = _Field('match_folder')
    paused = _Field('paused')
    unique_name = _Field('unique_name')
    subnet_label = _Field('subnet_label')
    subnet_type = _Field('subnet_type')
    throttles = _Field('throttles')
    throttled_bandwidth = _Field('throttled_bandwidth')
    throttled_bandwidth_current = _Field('throttled_bandwidth_current')
    throttled_bandwidth_kbps = _Field('throttled_bandwidth_kbps')
    throttled_bandwidth_current_kbps = _Field('throttled_bandwidth_current_kbps')
    subnet_network = _Field('subnet_network')
    subnet_netmask = _Field('subnet_netmask')
    volume_collection_list = _Field('volume_collection_list')
    volume_collection_list_count = _Field('volume_collection_list_count')
    volume_list = _Field('volume_list')
    volume_list_count = _Field('volume_list_count')
    replication_direction = _Field('replication_direction')


class InitiatorGroupFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    access_protocol = _Field('access_protocol')
    host_type = _Field('host_type')
    fc_tdz_ports = _Field('fc_tdz_ports')
    target_subnets = _Field('target_subnets')
    iscsi_initiators = _Field('iscsi_initiators')
    fc_initiators = _Field('fc_initiators')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    vp_override = _Field('vp_override')
    app_uuid = _Field('app_uuid')
    volume_count = _Field('volume_count')
    volume_list = _Field('volume_list')
    num_connections = _Field('num_connections')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)


class WitnessFields():
    id = _Field('id')
    username = _Field('username')
    password = _Field('password')
    host = _Field('host')
    port = _Field('port')
    secure_mode = _Field('secure_mode')
    auto_switchover_messages = _Field('auto_switchover_messages')


class SubnetFields():
    id = _Field('id')
    name = _Field('name')
    network = _Field('network')
    netmask = _Field('netmask')
    type = _Field('type')
    allow_iscsi = _Field('allow_iscsi')
    allow_group = _Field('allow_group')
    discovery_ip = _Field('discovery_ip')
    mtu = _Field('mtu')
    netzone_type = _Field('netzone_type')
    vlan_id = _Field('vlan_id')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    failover = _Field('failover')
    failover_enable_time = _Field('failover_enable_time')


class GroupFields():
    id = _Field('id')
    name = _Field('name')
    smtp_server = _Field('smtp_server')
    smtp_port = _Field('smtp_port')
    smtp_auth_enabled = _Field('smtp_auth_enabled')
    smtp_auth_username = _Field('smtp_auth_username')
    smtp_auth_password = _Field('smtp_auth_password')
    smtp_encrypt_type = _Field('smtp_encrypt_type')
    autosupport_enabled = _Field('autosupport_enabled')
    allow_analytics_gui = _Field('allow_analytics_gui')
    allow_support_tunnel = _Field('allow_support_tunnel')
    proxy_server = _Field('proxy_server')
    proxy_port = _Field('proxy_port')
    proxy_username = _Field('proxy_username')
    proxy_password = _Field('proxy_password')
    alert_to_email_addrs = _Field('alert_to_email_addrs')
    send_alert_to_support = _Field('send_alert_to_support')
    alert_from_email_addr = _Field('alert_from_email_addr')
    alert_min_level = _Field('alert_min_level')
    isns_enabled = _Field('isns_enabled')
    isns_server = _Field('isns_server')
    isns_port = _Field('isns_port')
    snmp_trap_enabled = _Field('snmp_trap_enabled')
    snmp_trap_host = _Field('snmp_trap_host')
    snmp_trap_port = _Field('snmp_trap_port')
    snmp_get_enabled = _Field('snmp_get_enabled')
    snmp_community = _Field('snmp_community')
    snmp_get_port = _Field('snmp_get_port')
    snmp_sys_contact = _Field('snmp_sys_contact')
    snmp_sys_location = _Field('snmp_sys_location')
    domain_name = _Field('domain_name')
    dns_servers = _Field('dns_servers')
    ntp_server = _Field('ntp_server')
    timezone = _Field('timezone')
    user_inactivity_timeout = _Field('user_inactivity_timeout')
    syslogd_enabled = _Field('syslogd_enabled')
    syslogd_server = _Field('syslogd_server')
    syslogd_port = _Field('syslogd_port')
    vvol_enabled = _Field('vvol_enabled')
    iscsi_enabled = _Field('iscsi_enabled')
    fc_enabled = _Field('fc_enabled')
    unique_name_enabled = _Field('unique_name_enabled')
    access_protocol_list = _Field('access_protocol_list')
    group_target_enabled = _Field('group_target_enabled')
    default_iscsi_target_scope = _Field('default_iscsi_target_scope')
    tdz_enabled = _Field('tdz_enabled')
    tdz_prefix = _Field('tdz_prefix')
    group_target_name = _Field('group_target_name')
    default_volume_reserve = _Field('default_volume_reserve')
    default_volume_warn_level = _Field('default_volume_warn_level')
    default_volume_limit = _Field('default_volume_limit')
    default_snap_reserve = _Field('default_snap_reserve')
    default_snap_warn_level = _Field('default_snap_warn_level')
    default_snap_limit = _Field('default_snap_limit')
    default_snap_limit_percent = _Field('default_snap_limit_percent')
    alarms_enabled = _Field('alarms_enabled')
    vss_validation_timeout = _Field('vss_validation_timeout')
    auto_switchover_enabled = _Field('auto_switchover_enabled')
    auto_switchover_messages = _Field('auto_switchover_messages')
    merge_state = _Field('merge_state')
    merge_group_name = _Field('merge_group_name')
    tlsv1_enabled = _Field('tlsv1_enabled')
    cc_mode_enabled = _Field('cc_mode_enabled')
    group_snapshot_ttl = _Field('group_snapshot_ttl')
    autoclean_unmanaged_snapshots_ttl_unit = _Field('autoclean_unmanaged_snapshots_ttl_unit')
    autoclean_unmanaged_snapshots_enabled = _Field('autoclean_unmanaged_snapshots_enabled')
    leader_array_name = _Field('leader_array_name')
    leader_array_serial = _Field('leader_array_serial')
    management_service_backup_array_name = _Field('management_service_backup_array_name')
    management_service_backup_status = _Field('management_service_backup_status')
    failover_mode = _Field('failover_mode')
    witness_status = _Field('witness_status')
    member_list = _Field('member_list')
    compressed_vol_usage_bytes = _Field('compressed_vol_usage_bytes')
    compressed_snap_usage_bytes = _Field('compressed_snap_usage_bytes')
    uncompressed_vol_usage_bytes = _Field('uncompressed_vol_usage_bytes')
    uncompressed_snap_usage_bytes = _Field('uncompressed_snap_usage_bytes')
    usable_capacity_bytes = _Field('usable_capacity_bytes')
    usage = _Field('usage')
    raw_capacity = _Field('raw_capacity')
    usable_cache_capacity = _Field('usable_cache_capacity')
    raw_cache_capacity = _Field('raw_cache_capacity')
    snap_usage_populated = _Field('snap_usage_populated')
    pending_deletes = _Field('pending_deletes')
    num_connections = _Field('num_connections')
    vol_compression_ratio = _Field('vol_compression_ratio')
    snap_compression_ratio = _Field('snap_compression_ratio')
    compression_ratio = _Field('compression_ratio')
    dedupe_ratio = _Field('dedupe_ratio')
    clone_ratio = _Field('clone_ratio')
    vol_thin_provisioning_ratio = _Field('vol_thin_provisioning_ratio')
    savings_ratio = _Field('savings_ratio')
    data_reduction_ratio = _Field('data_reduction_ratio')
    savings_dedupe = _Field('savings_dedupe')
    savings_compression = _Field('savings_compression')
    savings_clone = _Field('savings_clone')
    savings_vol_thin_provisioning = _Field('savings_vol_thin_provisioning')
    savings_data_reduction = _Field('savings_data_reduction')
    savings = _Field('savings')
    free_space = _Field('free_space')
    unused_reserve_bytes = _Field('unused_reserve_bytes')
    usage_valid = _Field('usage_valid')
    space_info_valid = _Field('space_info_valid')
    version_current = _Field('version_current')
    version_target = _Field('version_target')
    version_rollback = _Field('version_rollback')
    update_state = _Field('update_state')
    update_start_time = _Field('update_start_time')
    update_end_time = _Field('update_end_time')
    update_array_names = _Field('update_array_names')
    update_progress_msg = _Field('update_progress_msg')
    update_error_code = _Field('update_error_code')
    update_downloading = _Field('update_downloading')
    update_download_error_code = _Field('update_download_error_code')
    update_download_start_time = _Field('update_download_start_time')
    update_download_end_time = _Field('update_download_end_time')
    iscsi_automatic_connection_method = _Field('iscsi_automatic_connection_method')
    iscsi_connection_rebalancing = _Field('iscsi_connection_rebalancing')
    repl_throttled_bandwidth = _Field('repl_throttled_bandwidth')
    repl_throttled_bandwidth_kbps = _Field('repl_throttled_bandwidth_kbps')
    repl_throttle_list = _Field('repl_throttle_list')
    volume_migration_status = _Field('volume_migration_status')
    array_unassign_migration_status = _Field('array_unassign_migration_status')
    data_rebalance_status = _Field('data_rebalance_status')
    scsi_vendor_id = _Field('scsi_vendor_id')
    encryption_config = _Field('encryption_config')
    last_login = _Field('last_login')
    num_snaps = _Field('num_snaps')
    num_snapcolls = _Field('num_snapcolls')
    date = _Field('date')
    login_banner_message = _Field('login_banner_message')
    login_banner_after_auth = _Field('login_banner_after_auth')
    login_banner_reset = _Field('login_banner_reset')
    snap_retn_meter_high = _Field('snap_retn_meter_high')
    snap_retn_meter_very_high = _Field('snap_retn_meter_very_high')


class AccessControlRecordFields():
    id = _Field('id')
    apply_to = _Field('apply_to')
    chap_user_id = _Field('chap_user_id')
    chap_user_name = _Field('chap_user_name')
    initiator_group_id = _Field('initiator_group_id')
    initiator_group_name = _Field('initiator_group_name')
    lun = _Field('lun')
    vol_id = _Field('vol_id')
    vol_name = _Field('vol_name')
    vol_agent_type = _Field('vol_agent_type')
    pe_id = _Field('pe_id')
    pe_name = _Field('pe_name')
    pe_lun = _Field('pe_lun')
    snap_id = _Field('snap_id')
    snap_name = _Field('snap_name')
    pe_ids = _Field('pe_ids')
    snapluns = _Field('snapluns')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    access_protocol = _Field('access_protocol')


class ShelfFields():
    id = _Field('id')
    array_name = _Field('array_name')
    array_id = _Field('array_id')
    partial_response_ok = _Field('partial_response_ok')
    chassis_type = _Field('chassis_type')
    ctrlrs = _Field('ctrlrs')
    serial = _Field('serial')
    model = _Field('model')
    model_ext = _Field('model_ext')
    chassis_sensors = _Field('chassis_sensors')
    psu_overall_status = _Field('psu_overall_status')
    fan_overall_status = _Field('fan_overall_status')
    temp_overall_status = _Field('temp_overall_status')
    disk_sets = _Field('disk_sets')
    activated = _Field('activated')
    driveset = _Field('driveset')
    force = _Field('force')
    accept_foreign = _Field('accept_foreign')
    accept_dedupe_impact = _Field('accept_dedupe_impact')
    last_request = _Field('last_request')


class FibreChannelInitiatorAliasFields():
    id = _Field('id')
    alias = _Field('alias')
    wwpn = _Field('wwpn')
    source = _Field('source')


class NetworkConfigFields():
    id = _Field('id')
    name = _Field('name')
    mgmt_ip = _Field('mgmt_ip')
    secondary_mgmt_ip = _Field('secondary_mgmt_ip')
    role = _Field('role')
    iscsi_automatic_connection_method = _Field('iscsi_automatic_connection_method')
    iscsi_connection_rebalancing = _Field('iscsi_connection_rebalancing')
    route_list = _Field('route_list')
    subnet_list = _Field('subnet_list')
    array_list = _Field('array_list')
    group_leader_array = _Field('group_leader_array')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    active_since = _Field('active_since')
    last_active = _Field('last_active')
    ignore_validation_mask = _Field('ignore_validation_mask')


class MasterKeyFields():
    id = _Field('id')
    name = _Field('name')
    passphrase = _Field('passphrase')
    new_passphrase = _Field('new_passphrase')
    active = _Field('active')
    purge_age = _Field('purge_age')


class ProtocolEndpointFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    state = _Field('state')
    serial_number = _Field('serial_number')
    target_name = _Field('target_name')
    group_specific_ids = _Field('group_specific_ids')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    num_connections = _Field('num_connections')
    num_iscsi_connections = _Field('num_iscsi_connections')
    num_fc_connections = _Field('num_fc_connections')
    access_control_records = _Field('access_control_records')
    iscsi_sessions = _Field('iscsi_sessions')
    fc_sessions = _Field('fc_sessions')
    access_protocol = _Field('access_protocol')


class PoolFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    capacity = _Field('capacity')
    usage = _Field('usage')
    savings = _Field('savings')
    savings_data_reduction = _Field('savings_data_reduction')
    savings_compression = _Field('savings_compression')
    savings_dedupe = _Field('savings_dedupe')
    savings_clone = _Field('savings_clone')
    savings_vol_thin_provisioning = _Field('savings_vol_thin_provisioning')
    reserve = _Field('reserve')
    unused_reserve = _Field('unused_reserve')
    free_space = _Field('free_space')
    cache_capacity = _Field('cache_capacity')
    pinnable_cache_capacity = _Field('pinnable_cache_capacity')
    pinned_cache_capacity = _Field('pinned_cache_capacity')
    dedupe_capacity_bytes = _Field('dedupe_capacity_bytes')
    dedupe_usage_bytes = _Field('dedupe_usage_bytes')
    savings_ratio = _Field('savings_ratio')
    data_reduction_ratio = _Field('data_reduction_ratio')
    compression_ratio = _Field('compression_ratio')
    dedupe_ratio = _Field('dedupe_ratio')
    clone_ratio = _Field('clone_ratio')
    vol_thin_provisioning_ratio = _Field('vol_thin_provisioning_ratio')
    snapcoll_count = _Field('snapcoll_count')
    snap_count = _Field('snap_count')
    array_count = _Field('array_count')
    vol_count = _Field('vol_count')
    array_list = _Field('array_list')
    unassigned_array_list = _Field('unassigned_array_list')
    vol_list = _Field('vol_list')
    pinned_vol_list = _Field('pinned_vol_list')
    folder_list = _Field('folder_list')
    force = _Field('force')
    usage_valid = _Field('usage_valid')
    uncompressed_vol_usage_bytes = _Field('uncompressed_vol_usage_bytes')
    uncompressed_snap_usage_bytes = _Field('uncompressed_snap_usage_bytes')
    all_flash = _Field('all_flash')
    dedupe_capable = _Field('dedupe_capable')
    dedupe_all_volumes_capable = _Field('dedupe_all_volumes_capable')
    dedupe_all_volumes = _Field('dedupe_all_volumes')
    is_default = _Field('is_default')


class UserFields():
    id = _Field('id')
    name = _Field('name')
    search_name = _Field('search_name')
    description = _Field('description')
    role_id = _Field('role_id')
    role = _Field('role')
    password = _Field('password')
    auth_password = _Field('auth_password')
    inactivity_timeout = _Field('inactivity_timeout')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    full_name = _Field('full_name')
    email_addr = _Field('email_addr')
    disabled = _Field('disabled')
    auth_lock = _Field('auth_lock')
    last_login = _Field('last_login')
    last_logout = _Field('last_logout')
    logged_in = _Field('logged_in')


class SpaceDomainFields():
    id = _Field('id')
    pool_id = _Field('pool_id')
    pool_name = _Field('pool_name')
    app_category_id = _Field('app_category_id')
    app_category_name = _Field('app_category_name')
    perf_policy_names = _Field('perf_policy_names')
    sample_rate = _Field('sample_rate')
    volume_count = _Field('volume_count')
    deduped_volume_count = _Field('deduped_volume_count')
    volumes = _Field('volumes')
    block_size = _Field('block_size')
    deduped = _Field('deduped')
    encrypted = _Field('encrypted')
    usage = _Field('usage')
    vol_logical_usage = _Field('vol_logical_usage')
    snap_logical_usage = _Field('snap_logical_usage')
    vol_mapped_usage = _Field('vol_mapped_usage')
    logical_dedupe_usage = _Field('logical_dedupe_usage')
    physical_dedupe_usage = _Field('physical_dedupe_usage')
    savings_compression = _Field('savings_compression')
    savings_dedupe = _Field('savings_dedupe')
    savings_clone = _Field('savings_clone')
    compressed_usage_bytes = _Field('compressed_usage_bytes')
    uncompressed_usage_bytes = _Field('uncompressed_usage_bytes')
    compression_ratio = _Field('compression_ratio')
    dedupe_ratio = _Field('dedupe_ratio')
    clone_ratio = _Field('clone_ratio')


class UserPolicyFields():
    id = _Field('id')
    allowed_attempts = _Field('allowed_attempts')
    min_length = _Field('min_length')
    upper = _Field('upper')
    lower = _Field('lower')
    digit = _Field('digit')
    special = _Field('special')
    previous_diff = _Field('previous_diff')
    no_reuse = _Field('no_reuse')
    max_sessions = _Field('max_sessions')


class AlarmFields():
    id = _Field('id')
    type = _Field('type')
    array = _Field('array')
    curr_onset_event_id = _Field('curr_onset_event_id')
    object_id = _Field('object_id')
    object_name = _Field('object_name')
    object_type = _Field('object_type')
    onset_time = _Field('onset_time')
    ack_time = _Field('ack_time')
    status = _Field('status')
    user_id = _Field('user_id')
    user_name = _Field('user_name')
    user_full_name = _Field('user_full_name')
    category = _Field('category')
    severity = _Field('severity')
    remind_every = _Field('remind_every')
    remind_every_unit = _Field('remind_every_unit')
    activity = _Field('activity')
    next_notification_time = _Field('next_notification_time')


class ControllerFields():
    id = _Field('id')
    name = _Field('name')
    array_name = _Field('array_name')
    array_id = _Field('array_id')
    partial_response_ok = _Field('partial_response_ok')
    serial = _Field('serial')
    hostname = _Field('hostname')
    support_address = _Field('support_address')
    support_netmask = _Field('support_netmask')
    support_nic = _Field('support_nic')
    power_status = _Field('power_status')
    fan_status = _Field('fan_status')
    temperature_status = _Field('temperature_status')
    power_supplies = _Field('power_supplies')
    fans = _Field('fans')
    temperature_sensors = _Field('temperature_sensors')
    partition_status = _Field('partition_status')
    ctrlr_side = _Field('ctrlr_side')
    state = _Field('state')
    nvme_cards_enabled = _Field('nvme_cards_enabled')
    nvme_cards = _Field('nvme_cards')
    asup_time = _Field('asup_time')


class FibreChannelSessionFields():
    id = _Field('id')
    initiator_info = _Field('initiator_info')
    target_info = _Field('target_info')


class VersionFields():
    name = _Field('name')
    software_version = _Field('software_version')


class VolumeFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    size = _Field('size')
    description = _Field('description')
    perfpolicy_name = _Field('perfpolicy_name')
    perfpolicy_id = _Field('perfpolicy_id')
    reserve = _Field('reserve')
    warn_level = _Field('warn_level')
    limit = _Field('limit')
    snap_reserve = _Field('snap_reserve')
    snap_warn_level = _Field('snap_warn_level')
    snap_limit = _Field('snap_limit')
    snap_limit_percent = _Field('snap_limit_percent')
    num_snaps = _Field('num_snaps')
    projected_num_snaps = _Field('projected_num_snaps')
    online = _Field('online')
    owned_by_group = _Field('owned_by_group')
    owned_by_group_id = _Field('owned_by_group_id')
    multi_initiator = _Field('multi_initiator')
    iscsi_target_scope = _Field('iscsi_target_scope')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    read_only = _Field('read_only')
    serial_number = _Field('serial_number')
    secondary_serial_number = _Field('secondary_serial_number')
    target_name = _Field('target_name')
    block_size = _Field('block_size')
    offline_reason = _Field('offline_reason')
    clone = _Field('clone')
    parent_vol_name = _Field('parent_vol_name')
    parent_vol_id = _Field('parent_vol_id')
    base_snap_name = _Field('base_snap_name')
    base_snap_id = _Field('base_snap_id')
    replication_role = _Field('replication_role')
    volcoll_name = _Field('volcoll_name')
    volcoll_id = _Field('volcoll_id')
    agent_type = _Field('agent_type')
    force = _Field('force')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    protection_type = _Field('protection_type')
    last_snap = _Field('last_snap')
    last_replicated_snap = _Field('last_replicated_snap')
    dest_pool_name = _Field('dest_pool_name')
    dest_pool_id = _Field('dest_pool_id')
    move_start_time = _Field('move_start_time')
    move_aborting = _Field('move_aborting')
    move_bytes_migrated = _Field('move_bytes_migrated')
    move_bytes_remaining = _Field('move_bytes_remaining')
    move_est_compl_time = _Field('move_est_compl_time')
    usage_valid = _Field('usage_valid')
    space_usage_level = _Field('space_usage_level')
    total_usage_bytes = _Field('total_usage_bytes')
    vol_usage_compressed_bytes = _Field('vol_usage_compressed_bytes')
    vol_usage_uncompressed_bytes = _Field('vol_usage_uncompressed_bytes')
    vol_usage_mapped_bytes = _Field('vol_usage_mapped_bytes')
    snap_usage_compressed_bytes = _Field('snap_usage_compressed_bytes')
    snap_usage_uncompressed_bytes = _Field('snap_usage_uncompressed_bytes')
    snap_usage_populated_bytes = _Field('snap_usage_populated_bytes')
    cache_pinned = _Field('cache_pinned')
    pinned_cache_size = _Field('pinned_cache_size')
    cache_needed_for_pin = _Field('cache_needed_for_pin')
    upstream_cache_pinned = _Field('upstream_cache_pinned')
    cache_policy = _Field('cache_policy')
    thinly_provisioned = _Field('thinly_provisioned')
    vol_state = _Field('vol_state')
    online_snaps = _Field('online_snaps')
    num_connections = _Field('num_connections')
    num_iscsi_connections = _Field('num_iscsi_connections')
    num_fc_connections = _Field('num_fc_connections')
    access_control_records = _Field('access_control_records')
    inherit_acl = _Field('inherit_acl')
    encryption_cipher = _Field('encryption_cipher')
    app_uuid = _Field('app_uuid')
    folder_id = _Field('folder_id')
    folder_name = _Field('folder_name')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    iscsi_sessions = _Field('iscsi_sessions')
    fc_sessions = _Field('fc_sessions')
    caching_enabled = _Field('caching_enabled')
    previously_deduped = _Field('previously_deduped')
    dedupe_enabled = _Field('dedupe_enabled')
    vpd_t10 = _Field('vpd_t10')
    vpd_ieee0 = _Field('vpd_ieee0')
    vpd_ieee1 = _Field('vpd_ieee1')
    app_category = _Field('app_category')
    limit_iops = _Field('limit_iops')
    limit_mbps = _Field('limit_mbps')
    needs_content_repl = _Field('needs_content_repl')
    content_repl_errors_found = _Field('content_repl_errors_found')
    last_content_snap_br_cg_uid = _Field('last_content_snap_br_cg_uid')
    last_content_snap_br_gid = _Field('last_content_snap_br_gid')
    last_content_snap_id = _Field('last_content_snap_id')
    cksum_last_verified = _Field('cksum_last_verified')
    pre_filter = _Field('pre_filter')
    avg_stats_last_5mins = _Field('avg_stats_last_5mins')
    srep_last_sync = _Field('srep_last_sync')
    srep_resync_percent = _Field('srep_resync_percent')


class ProtectionScheduleFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    volcoll_or_prottmpl_type = _Field('volcoll_or_prottmpl_type')
    volcoll_or_prottmpl_id = _Field('volcoll_or_prottmpl_id')
    period = _Field('period')
    period_unit = _Field('period_unit')
    at_time = _Field('at_time')
    until_time = _Field('until_time')
    days = _Field('days')
    num_retain = _Field('num_retain')
    downstream_partner = _Field('downstream_partner')
    downstream_partner_name = _Field('downstream_partner_name')
    downstream_partner_id = _Field('downstream_partner_id')
    upstream_partner_name = _Field('upstream_partner_name')
    upstream_partner_id = _Field('upstream_partner_id')
    replicate_every = _Field('replicate_every')
    num_retain_replica = _Field('num_retain_replica')
    repl_alert_thres = _Field('repl_alert_thres')
    snap_verify = _Field('snap_verify')
    skip_db_consistency_check = _Field('skip_db_consistency_check')
    disable_appsync = _Field('disable_appsync')
    schedule_type = _Field('schedule_type')
    active = _Field('active')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    last_mod_sched_time = _Field('last_mod_sched_time')
    last_replicated_snapcoll_name = _Field('last_replicated_snapcoll_name')
    last_replicated_snapcoll_id = _Field('last_replicated_snapcoll_id')
    last_replicated_at_time = _Field('last_replicated_at_time')
    last_snap_time = _Field('last_snap_time')
    next_snap_time = _Field('next_snap_time')
    next_repl_snap_time = _Field('next_repl_snap_time')
    snap_counter = _Field('snap_counter')
    sched_owner_id = _Field('sched_owner_id')
    sched_owner_name = _Field('sched_owner_name')
    last_config_change_time = _Field('last_config_change_time')
    vol_status_list = _Field('vol_status_list')
    sync_repl_vol_status_list = _Field('sync_repl_vol_status_list')
    use_downstream_for_DR = _Field('use_downstream_for_DR')


class ProtectionTemplateFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    repl_priority = _Field('repl_priority')
    app_sync = _Field('app_sync')
    app_server = _Field('app_server')
    app_id = _Field('app_id')
    app_cluster_name = _Field('app_cluster_name')
    app_service_name = _Field('app_service_name')
    vcenter_hostname = _Field('vcenter_hostname')
    vcenter_username = _Field('vcenter_username')
    vcenter_password = _Field('vcenter_password')
    agent_hostname = _Field('agent_hostname')
    agent_username = _Field('agent_username')
    agent_password = _Field('agent_password')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    schedule_list = _Field('schedule_list')


class VolumeCollectionFields():
    id = _Field('id')
    prottmpl_id = _Field('prottmpl_id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    repl_priority = _Field('repl_priority')
    pol_owner_name = _Field('pol_owner_name')
    replication_type = _Field('replication_type')
    synchronous_replication_type = _Field('synchronous_replication_type')
    synchronous_replication_state = _Field('synchronous_replication_state')
    app_sync = _Field('app_sync')
    app_server = _Field('app_server')
    app_id = _Field('app_id')
    app_cluster_name = _Field('app_cluster_name')
    app_service_name = _Field('app_service_name')
    vcenter_hostname = _Field('vcenter_hostname')
    vcenter_username = _Field('vcenter_username')
    vcenter_password = _Field('vcenter_password')
    agent_hostname = _Field('agent_hostname')
    agent_username = _Field('agent_username')
    agent_password = _Field('agent_password')
    creation_time = _Field('creation_time')
    last_modified_time = _Field('last_modified_time')
    volume_list = _Field('volume_list')
    downstream_volume_list = _Field('downstream_volume_list')
    upstream_volume_list = _Field('upstream_volume_list')
    volume_count = _Field('volume_count')
    cache_pinned_volume_list = _Field('cache_pinned_volume_list')
    last_snapcoll = _Field('last_snapcoll')
    snapcoll_count = _Field('snapcoll_count')
    schedule_list = _Field('schedule_list')
    replication_partner = _Field('replication_partner')
    last_replicated_snapcoll = _Field('last_replicated_snapcoll')
    last_replicated_snapcoll_list = _Field('last_replicated_snapcoll_list')
    protection_type = _Field('protection_type')
    lag_time = _Field('lag_time')
    is_standalone_volcoll = _Field('is_standalone_volcoll')
    total_repl_bytes = _Field('total_repl_bytes')
    repl_bytes_transferred = _Field('repl_bytes_transferred')
    is_handing_over = _Field('is_handing_over')
    handover_replication_partner = _Field('handover_replication_partner')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    srep_last_sync = _Field('srep_last_sync')
    srep_resync_percent = _Field('srep_resync_percent')


class FibreChannelInterfaceFields():
    id = _Field('id')
    array_name_or_serial = _Field('array_name_or_serial')
    partial_response_ok = _Field('partial_response_ok')
    controller_name = _Field('controller_name')
    fc_port_id = _Field('fc_port_id')
    name = _Field('name')
    wwnn = _Field('wwnn')
    wwpn = _Field('wwpn')
    peerzone = _Field('peerzone')
    online = _Field('online')
    firmware_version = _Field('firmware_version')
    logical_port_number = _Field('logical_port_number')
    fc_port_name = _Field('fc_port_name')
    bus_location = _Field('bus_location')
    slot = _Field('slot')
    orientation = _Field('orientation')
    port = _Field('port')
    link_info = _Field('link_info')
    fabric_info = _Field('fabric_info')


class SnapshotFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    size = _Field('size')
    vol_name = _Field('vol_name')
    pool_name = _Field('pool_name')
    vol_id = _Field('vol_id')
    snap_collection_name = _Field('snap_collection_name')
    snap_collection_id = _Field('snap_collection_id')
    online = _Field('online')
    writable = _Field('writable')
    offline_reason = _Field('offline_reason')
    expiry_time = _Field('expiry_time')
    expiry_after = _Field('expiry_after')
    origin_name = _Field('origin_name')
    is_replica = _Field('is_replica')
    is_unmanaged = _Field('is_unmanaged')
    is_manually_managed = _Field('is_manually_managed')
    replication_status = _Field('replication_status')
    access_control_records = _Field('access_control_records')
    serial_number = _Field('serial_number')
    target_name = _Field('target_name')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    schedule_name = _Field('schedule_name')
    schedule_id = _Field('schedule_id')
    app_uuid = _Field('app_uuid')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    new_data_valid = _Field('new_data_valid')
    new_data_compressed_bytes = _Field('new_data_compressed_bytes')
    new_data_uncompressed_bytes = _Field('new_data_uncompressed_bytes')
    agent_type = _Field('agent_type')
    vpd_t10 = _Field('vpd_t10')
    vpd_ieee0 = _Field('vpd_ieee0')
    vpd_ieee1 = _Field('vpd_ieee1')
    force = _Field('force')


class KeyManagerFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    hostname = _Field('hostname')
    port = _Field('port')
    protocol = _Field('protocol')
    username = _Field('username')
    password = _Field('password')
    active = _Field('active')
    status = _Field('status')
    vendor = _Field('vendor')


class NetworkInterfaceFields():
    id = _Field('id')
    array_name_or_serial = _Field('array_name_or_serial')
    partial_response_ok = _Field('partial_response_ok')
    array_id = _Field('array_id')
    controller_name = _Field('controller_name')
    controller_id = _Field('controller_id')
    name = _Field('name')
    mac = _Field('mac')
    is_present = _Field('is_present')
    link_speed = _Field('link_speed')
    link_status = _Field('link_status')
    mtu = _Field('mtu')
    port = _Field('port')
    slot = _Field('slot')
    max_link_speed = _Field('max_link_speed')
    nic_type = _Field('nic_type')
    ip_list = _Field('ip_list')
